<?php

namespace FiloBlu\Venistar\Esb\Consumer;

use FiloBlu\Esb\Api\Data\MessageInterface;
use FiloBlu\Esb\Api\Data\MessageInterfaceFactory;
use FiloBlu\Esb\Api\Data\ObjectTypeDescriptorInterface;
use FiloBlu\Esb\Api\Data\StatusInterface;
use FiloBlu\Esb\Api\Data\StatusInterfaceFactory;
use FiloBlu\Esb\Core\Expression\EvaluatorInterfaceFactory;
use FiloBlu\Esb\Core\Extractor\ObjectTypeFromMessage;
use FiloBlu\Esb\Framework\Consumer\AbstractConsumer;
use FiloBlu\Flow\Model\Connector\Ftp as FlowFtpConnector;
use FiloBlu\Venistar\Helper\Api as ApiHelper;
use Psr\Log\LoggerInterface;

/**
 *
 */
class SendRmaToVenistar extends AbstractConsumer
{
    /**
     * @var ApiHelper
     */
    protected $apiHelper;

    /**
     * @var \FiloBlu\Flow\Model\Connector\Ftp
     */
    protected $flowFtpConnector;

    /**
     * @param \FiloBlu\Esb\Core\Expression\EvaluatorInterfaceFactory $evaluatorFactory
     * @param \FiloBlu\Esb\Api\Data\ObjectTypeDescriptorInterface $objectTypeDescriptor
     * @param \FiloBlu\Esb\Api\Data\MessageInterfaceFactory $messageFactory
     * @param \FiloBlu\Esb\Core\Extractor\ObjectTypeFromMessage $objectTypeFromMessage
     * @param \FiloBlu\Esb\Api\Data\StatusInterfaceFactory $statusFactory
     * @param \Psr\Log\LoggerInterface $logger
     * @param \FiloBlu\Venistar\Helper\Api $apiHelper
     * @param \FiloBlu\Flow\Model\Connector\Ftp $flowFtpConnector
     */
    public function __construct(
        EvaluatorInterfaceFactory $evaluatorFactory,
        ObjectTypeDescriptorInterface $objectTypeDescriptor,
        MessageInterfaceFactory $messageFactory,
        ObjectTypeFromMessage $objectTypeFromMessage,
        StatusInterfaceFactory $statusFactory,
        LoggerInterface $logger,
        ApiHelper $apiHelper,
        FlowFtpConnector $flowFtpConnector
    ) {
        parent::__construct(
            $evaluatorFactory,
            $objectTypeDescriptor,
            $messageFactory,
            $objectTypeFromMessage,
            $statusFactory,
            $logger
        );
        $this->apiHelper = $apiHelper;
        $this->flowFtpConnector = $flowFtpConnector;
    }

    /**
     * @param \FiloBlu\Esb\Api\Data\MessageInterface $message
     * @return \FiloBlu\Esb\Api\Data\MessageInterface
     * @throws \FiloBlu\Esb\Core\Exception\NonRecoverableException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function consume(MessageInterface $message): MessageInterface
    {
        $rma = $this->loadObject($message);
        $rma_status = $rma->getStatus();
        $parameters = $this->getConfiguration()->getParameters();
        $this->flowFtpConnector->setConfig($parameters->getDataByKey('ftp'));
        $json = $this->apiHelper->getRmaDataByApi($rma->getEntityId(), $parameters->getDataByKey('token'));
        $filename = 'rma_' . $rma->getIncrementId() . '_' . $rma_status . '_' . date('YmdHis') . '.json';
        $filename_path = $this->flowFtpConnector->localDir . DIRECTORY_SEPARATOR . $filename;
        if (!$json) {
            $this->getStatus()->setCode(StatusInterface::ERROR);
            $this->getStatus()->appendOutputData("Can't generate JSON from rma {$rma->getIncrementId()}");
            return $this->messageFactory->create();
        }

        $fopen = fopen($filename_path, 'wb');
        fwrite($fopen, $json);
        fclose($fopen);
        $fileUploaded = $this->flowFtpConnector->sendResource($filename_path);

        if (!$fileUploaded) {
            $this->getStatus()->setCode(StatusInterface::ERROR);
            $this->getStatus()->appendOutputData("File upload on FTP failed for rma {$rma->getIncrementId()}");
            return $this->messageFactory->create();
        }

        $this->getStatus()->setCode(StatusInterface::SUCCESS);
        $this->getStatus()->appendOutputData("File uploaded on FTP successfully for rma {$rma->getIncrementId()}");
        return $this->messageFactory->create();
    }

    /**
     * @return string
     */
    public function getName(): string
    {
        return 'venistar_rma_sender';
    }

    /**
     * @return string
     */
    public function getUniqueName(): string
    {
        return 'FiloBlu_Venistar::rma_sender';
    }
}
